/*@@
  @file       NewRad.F
  @date       September 2001
  @author     Miguel Alcubierre
  @desc
              Routine to apply an outgoing wave boundary condition.
  @enddesc
@@*/

#include "cctk.h"
#include "cctk_Parameters.h"
#include "cctk_Functions.h"

      subroutine newrad(doBC,nx,ny,nz,dx,dy,dz,x,y,z,r,
     .var,source,var0,v0,power)

      implicit none

      DECLARE_CCTK_PARAMETERS
      DECLARE_CCTK_FUNCTIONS

      logical doBC(6)

      integer i,j,k
      integer nx,ny,nz
      integer sx,sy,sz
      integer svx,svy,svz
      integer ip,jp,kp

      CCTK_REAL v0,var0,power
      CCTK_REAL dx,dy,dz
      CCTK_REAL idx,idy,idz
      CCTK_REAL vx,vy,vz
      CCTK_REAL derivx,derivy,derivz
      CCTK_REAL zero,half,one
      CCTK_REAL aux,rp,rpi

      CCTK_REAL, dimension(nx,ny,nz) :: x,y,z,r
      CCTK_REAL, dimension(nx,ny,nz) :: var,source


c     *******************
c     ***   NUMBERS   ***
c     *******************

      zero = 0.0D0
      half = 0.5D0
      one  = 1.0D0

      idx = one/dx
      idy = one/dy
      idz = one/dz


c     *******************
c     ***   6 FACES   ***
c     *******************

c     Lower x.

      if (doBC(1)) then

         i = 1
         sx = -1; sy = 0; sz = 0

         do k=2,nz-1
            do j=2,ny-1
# include "include/newrad.h"
            end do
         end do

      end if

c     Upper x.

      if (doBC(2)) then

         i = nx
         sx = +1; sy = 0; sz = 0

         do k=2,nz-1
            do j=2,ny-1
# include "include/newrad.h"
            end do
         end do

      end if

c     Lower y.

      if (doBC(3)) then

         j = 1
         sx = 0; sy = -1; sz = 0

         do k=2,nz-1
            do i=2,nx-1
# include "include/newrad.h"
            end do
         end do

      end if

c     Upper y.

      if (doBC(4)) then

         j = ny
         sx = 0; sy = +1; sz = 0

         do k=2,nz-1
            do i=2,nx-1
# include "include/newrad.h"
            end do
         end do

      end if

c     Lower z.

      if (doBC(5)) then

         k = 1
         sx = 0; sy = 0; sz = -1

         do j=2,ny-1
            do i=2,nx-1
# include "include/newrad.h"
            end do
         end do

      end if

c     Upper z.

      if (doBC(6)) then

         k = nz
         sx = 0; sy = 0; sz = +1

         do j=2,ny-1
            do i=2,nx-1
# include "include/newrad.h"
            end do
         end do

      end if


c     ********************
c     ***   12 EDGES   ***
c     ********************

c     Lower x - Lower y.

      if (doBC(1).and.doBC(3)) then

         i = 1; j = 1
         sx = -1; sy = -1; sz = 0

         do k=2,nz-1
# include "include/newrad.h"
         end do

      end if

c     Lower x - Upper y.

      if (doBC(1).and.doBC(4)) then

         i = 1; j = ny
         sx = -1; sy = +1; sz = 0

         do k=2,nz-1
# include "include/newrad.h"
         end do

      end if

c     Upper x - Lower y.

      if (doBC(2).and.doBC(3)) then

         i = nx; j = 1
         sx = +1; sy = -1; sz = 0

         do k=2,nz-1
# include "include/newrad.h"
         end do

      end if

c     Upper x - Upper y.

      if (doBC(2).and.doBC(4)) then

         i = nx; j = ny
         sx = +1; sy = +1; sz = 0

         do k=2,nz-1
# include "include/newrad.h"
         end do

      end if

c     Lower x - Lower z.

      if (doBC(1).and.doBC(5)) then

         i = 1; k = 1
         sx = -1; sy = 0; sz = -1

         do j=2,ny-1
# include "include/newrad.h"
         end do

      end if

c     Lower x - Upper z.

      if (doBC(1).and.doBC(6)) then

         i = 1; k = nz
         sx = -1; sy = 0; sz = +1

         do j=2,ny-1
# include "include/newrad.h"
         end do

      end if

c     Upper x - Lower z.

      if (doBC(2).and.doBC(5)) then

         i = nx; k = 1
         sx = +1; sy = 0; sz = -1

         do j=2,ny-1
# include "include/newrad.h"
         end do

      end if

c     Upper x - Upper z.

      if (doBC(2).and.doBC(6)) then

         i = nx; k = nz
         sx = +1; sy = 0; sz = +1

         do j=2,ny-1
# include "include/newrad.h"
         end do

      end if

c     Lower y - Lower z.

      if (doBC(3).and.doBC(5)) then

         j = 1; k = 1
         sx = 0; sy = -1; sz = -1

         do i=2,nx-1
# include "include/newrad.h"
         end do

      end if

c     Lower y - Upper z.

      if (doBC(3).and.doBC(6)) then

         j = 1; k = nz
         sx = 0; sy = -1; sz = +1

         do i=2,nx-1
# include "include/newrad.h"
         end do

      end if

c     Upper y - Lower z.

      if (doBC(4).and.doBC(5)) then

         j = ny; k = 1
         sx = 0; sy = +1; sz = -1

         do i=2,nx-1
# include "include/newrad.h"
         end do

      end if

c     Upper y - Upper z.

      if (doBC(4).and.doBC(6)) then

         j = ny; k = nz
         sx = 0; sy = +1; sz = +1

         do i=2,nx-1
# include "include/newrad.h"
         end do

      end if


c     *********************
c     ***   8 CORNERS   ***
c     *********************

c     Lower x - Lower y - Lower z.

      if (doBC(1).and.doBC(3).and.doBC(5)) then

         i = 1; j = 1; k = 1
         sx = -1; sy = -1; sz = -1

# include "include/newrad.h"

      end if

c     Lower x - Lower y - Upper z.

      if (doBC(1).and.doBC(3).and.doBC(6)) then

         i = 1; j = 1; k = nz
         sx = -1; sy = -1; sz = +1

# include "include/newrad.h"

      end if

c     Lower x - Upper y - Lower z.

      if (doBC(1).and.doBC(4).and.doBC(5)) then

         i = 1; j = ny; k = 1
         sx = -1; sy = +1; sz = -1

# include "include/newrad.h"

      end if

c     Lower x - Upper y - Upper z.

      if (doBC(1).and.doBC(4).and.doBC(6)) then

         i = 1; j = ny; k = nz
         sx = -1; sy = +1; sz = +1

# include "include/newrad.h"

      end if

c     Upper x - Lower y - Lower z.

      if (doBC(2).and.doBC(3).and.doBC(5)) then

         i = nx; j = 1; k = 1
         sx = +1; sy = -1; sz = -1

# include "include/newrad.h"

      end if

c     Upper x - Lower y - Upper z.

      if (doBC(2).and.doBC(3).and.doBC(6)) then

         i = nx; j = 1; k = nz
         sx = +1; sy = -1; sz = +1

# include "include/newrad.h"

      end if

c     Upper x - Upper y - Lower z.

      if (doBC(2).and.doBC(4).and.doBC(5)) then

         i = nx; j = ny; k = 1
         sx = +1; sy = +1; sz = -1

# include "include/newrad.h"

      end if

c     Upper x - Upper y - Upper z.

      if (doBC(2).and.doBC(4).and.doBC(6)) then

         i = nx; j = ny; k = nz
         sx = +1; sy = +1; sz = +1

# include "include/newrad.h"

      end if


c     ***************
c     ***   END   ***
c     ***************

      end subroutine NewRad
