c/*@@
c  @file    Sources.F
c  @date
c  @author  Gabrielle Allen + Miguel Alcubierre
c  @desc 
c     Construct sources for BS Evolution
c  @enddesc 
c@@*/

#include "cctk.h"
#include "cctk_Parameters.h"
#include "cctk_Arguments.h"
#include "cctk_Functions.h"

c  This can be useful:
#define INSOURCES
#ifdef OPT
 OPT is defined twice.  ADM_BSSN optimization will probably break code!
#endif


c     Marching plane optimizations added by David Rideout.
c     Deactivate by commenting out the next line:

c #define OPT


      subroutine ADM_BSSN_Sources(CCTK_ARGUMENTS)

      USE ADM_BSSN_Scalars

      implicit none

      DECLARE_CCTK_ARGUMENTS
      DECLARE_CCTK_PARAMETERS
      DECLARE_CCTK_FUNCTIONS

      logical :: center
c     logical :: upwind,upwind1

c     logical, allocatable, dimension (:,:,:) :: doupwind1x
c     logical, allocatable, dimension (:,:,:) :: doupwind1y
c     logical, allocatable, dimension (:,:,:) :: doupwind1z

      logical, allocatable, dimension (:,:,:) :: docenterx
      logical, allocatable, dimension (:,:,:) :: docentery
      logical, allocatable, dimension (:,:,:) :: docenterz
      logical, allocatable, dimension (:,:,:) :: force_spatial_second_order

      integer :: i,j,k,nx,ny,nz,imin(3),imax(3)
      integer :: sx,sy,sz,ssx,ssy,ssz
      integer :: wx,wy,wz
      integer :: ierr

      CCTK_REAL :: third,twothird,sixth
      CCTK_REAL :: half,pi,fourpi,eightpi
      CCTK_REAL :: BETAX_TEMP,BETAY_TEMP,BETAZ_TEMP
      CCTK_REAL :: ham_atten

      logical, allocatable, dimension (:,:,:) :: bigmask

      CCTK_INT  :: symtable
      CCTK_INT, dimension(1:6) :: symbnd
      logical,  dimension(1:6) :: doBC

#include "CactusEinstein/ADMMacros/src/macro/ADM_Spacing_declare.h"

c     Macro declarations.

#include "macro/PHISOURCES_declare.h"
#include "macro/BSGSOURCES_declare.h"
#include "macro/TRKSOURCES_declare.h"
#include "macro/ASOURCES_declare.h"
#include "macro/GAMMASOURCES_declare.h"

#include "macro/LIEPHI_declare.h"
#include "macro/LIEBSG_declare.h"
#include "macro/LIEK_declare.h"
#include "macro/LIEA_declare.h"
#include "macro/SHIFTGAMMA_declare.h"

c     Temporaries for the Hisaaki adjustments; could somebody
c     give these better names please?!

      CCTK_REAL aux, aux1, aux2, aux3

c     Extras for optimization.

#ifdef OPT

      integer :: km,kc,kp,kpp,ks,kss,ksss
      integer :: allocstat

      CCTK_REAL, allocatable, dimension (:,:,:,:)  :: lg
      CCTK_REAL, allocatable, dimension (:,:,:,:)  :: lA

c     Allocate optimization arrays for z-slices.

      allocate (lg(6,cctk_lsh(1),cctk_lsh(2),5), STAT=allocstat)
      allocate (lA(6,cctk_lsh(1),cctk_lsh(2),5), STAT=allocstat)

#endif

c========================================================================
c
c     Numbers
c
c========================================================================

      half     = 0.5D0
      third    = 1.0d0/3.0d0
      twothird = 2.0d0*third
      sixth    = 1.0d0/6.0d0

      pi       = acos(-1.0d0)
      fourpi   = 4.0D0*pi
      eightpi  = 8.0D0*pi


c========================================================================
c
c     Grid parameters.
c
c========================================================================

      symtable = SymmetryTableHandleForGrid (cctkGH)
      if (symtable < 0) call CCTK_WARN (0, "internal error")
      call Util_TableGetIntArray (ierr, int(symtable), 6, symbnd, "symmetry_handle")
      if (ierr /= 6) call CCTK_WARN (0, "internal error")
      doBC = cctk_bbox /= 0 .and. symbnd < 0
      if (periodic.ne.0 .and. periodic_x.ne.0) then
         doBC(1) = .false.
         doBC(2) = .false.
      end if
      if (periodic.ne.0 .and. periodic_y.ne.0) then
         doBC(3) = .false.
         doBC(4) = .false.
      end if
      if (periodic.ne.0 .and. periodic_z.ne.0) then
         doBC(5) = .false.
         doBC(6) = .false.
      end if

      nx = cctk_lsh(1)
      ny = cctk_lsh(2)
      nz = cctk_lsh(3)

      where (doBC(1:5:2))
         imin = 2
      elsewhere
         imin = 1 + cctk_nghostzones
      end where
      where (doBC(2:6:2))
         imax = cctk_lsh-1
      elsewhere
         imax = cctk_lsh - cctk_nghostzones
      end where
      

#include "CactusEinstein/ADMMacros/src/macro/ADM_Spacing.h"

c========================================================================
c
c     Initial stuff.
c
c========================================================================

c     Ghost zones for Cartoon.

      if (cartoon==1) then
         stencily = cctk_nghostzones(2)
      else
         stencily = 1
      end if

c     Find bigmask. The purpose of the array "bigmask" is to mark
c     points that have good data with 1, and points that do not with 0.
c     It plays the same role as the excision mask except that it extends
c     one grid point beyond the boundary.  This is because for second
c     order upwind we need to check if we have good data two grid points
c     away.  Since I wanted a general mechanism that could work in all
c     interior points (even those one point away from the boundaries),
c     defining this large array seemed the easiest way to go about it.
c     But notice that all of this is only needed if we have a non-zero
c     shift.  Otherwise everything is centered differences and we are OK.

      if (shift_state .ne. 0) then

         allocate(bigmask(0:nx+1,0:ny+1,0:nz+1))

         if (excise.eq.1) then

            if (use_mask.eq.0) call CCTK_WARN (0, "internal error")
            bigmask(1:nx,1:ny,1:nz) = abs(emask)<0.01
            ADM_BS_mask = emask

         else

            bigmask = .false.

         end if

         bigmask(0,:,:) = .true.
         bigmask(:,0,:) = .true.
         bigmask(:,:,0) = .true.

         bigmask(nx+1,:,:) = .true.
         bigmask(:,ny+1,:) = .true.
         bigmask(:,:,nz+1) = .true.

      end if

c     Find out type of advection.

      center = CCTK_EQUALS(advection,'center')

c     upwind = CCTK_EQUALS(advection,'upwind1').or.
c    .         CCTK_EQUALS(advection,'upwind2')

c     Fill in the values of the logical arrays "doupwind1*" and
c     "docenter*".  These arrays will have a "true" value at points
c     where we will use first order upwind or centered differences
c     repectively (either because we want to, or because we can not use
c     second order upwind), and a "false" value where we will use second
c     order upwind.

      if (shift_state .ne. 0) then

c        allocate(doupwind1x(nx,ny,nz),doupwind1y(nx,ny,nz),doupwind1z(nx,ny,nz))
         allocate(docenterx(nx,ny,nz),docentery(nx,ny,nz),docenterz(nx,ny,nz))

c        upwind1 = CCTK_EQUALS(advection,'upwind1')

         if (center) then
            docenterx = center
            docentery = center
            docenterz = center
         else
            do k=2,nz-1
               do j=1+stencily,ny-stencily
                  do i=2,nx-1

                     sx = 1; sy = 1; sz = 1
                     if (betax(i,j,k) < 0) sx = -1
                     if (betay(i,j,k) < 0) sy = -1
                     if (betaz(i,j,k) < 0) sz = -1

c                 doupwind1x(i,j,k) = upwind1.or.bigmask(i+2*sx,j,k)
c                 doupwind1y(i,j,k) = upwind1.or.bigmask(i,j+2*sy,k)
c                 doupwind1z(i,j,k) = upwind1.or.bigmask(i,j,k+2*sz)

                     docenterx(i,j,k) = bigmask(i+2*sx,j,k)
                     docentery(i,j,k) = bigmask(i,j+2*sy,k)
                     docenterz(i,j,k) = bigmask(i,j,k+2*sz)

                  end do
               end do
            end do
         end if

      end if

c     Now set the local_spatial_order depending on the position
c     relative to the boundaries or the excision region

      allocate (force_spatial_second_order(nx,ny,nz))
      force_spatial_second_order = .FALSE.

      if (spatial_order > 2) then
         do k=2,nz-1
            do j=1+stencily,ny-stencily
               do i=2,nx-1
                  if ((i < 3).or.(i > cctk_lsh(1) - 2).or.
     .                (j < 3).or.(j > cctk_lsh(2) - 2).or.
     .                (k < 3).or.(k > cctk_lsh(3) - 2) ) then
                     force_spatial_second_order(i,j,k) = .TRUE.
                  else if ( excise /= 0 .and. use_mask /= 0 ) then
                     if (minval(emask(i-2:i+2,j-2:j+2,k-2:k+2)) < 0.75d0) then
                        force_spatial_second_order(i,j,k) = .TRUE.
                     end if
                  end if
               end do
            end do
         end do
      end if

c======================================================================
c
c     Construct sources for conformal factor and metric.
c
c======================================================================

#ifdef OPT
#define XX 1
#define XY 2
#define XZ 3
#define YY 4
#define YZ 5
#define ZZ 6

c     Initialize arrays for planes.

      lg(XX,:,:,2:4) = ADM_BS_gxx(:,:,1:3)
      lg(XY,:,:,2:4) = ADM_BS_gxy(:,:,1:3)
      lg(XZ,:,:,2:4) = ADM_BS_gxz(:,:,1:3)
      lg(YY,:,:,2:4) = ADM_BS_gyy(:,:,1:3)
      lg(YZ,:,:,2:4) = ADM_BS_gyz(:,:,1:3)
      lg(ZZ,:,:,2:4) = ADM_BS_gzz(:,:,1:3)

      lA(XX,:,:,2:4) = ADM_BS_Axx(:,:,1:3)
      lA(XY,:,:,2:4) = ADM_BS_Axy(:,:,1:3)
      lA(XZ,:,:,2:4) = ADM_BS_Axz(:,:,1:3)
      lA(YY,:,:,2:4) = ADM_BS_Ayy(:,:,1:3)
      lA(YZ,:,:,2:4) = ADM_BS_Ayz(:,:,1:3)
      lA(ZZ,:,:,2:4) = ADM_BS_Azz(:,:,1:3)
#endif

c     Loop over z.
      do k=imin(3),imax(3)

#ifdef OPT
c        Shift location in 'l' 'slabs'.

         km  = 1 + mod(k-1, 5)
         kc  = 1 + mod(k,   5)
         kp  = 1 + mod(k+1, 5)
         kpp = 1 + mod(k+2, 5)

c        If not the last k, put next slice into lg.

         if (k.ne.nz-1) then 
            lg(XX,:,:,kpp) = ADM_BS_gxx(:,:,k+2)
            lg(XY,:,:,kpp) = ADM_BS_gxy(:,:,k+2)
            lg(XZ,:,:,kpp) = ADM_BS_gxz(:,:,k+2)
            lg(YY,:,:,kpp) = ADM_BS_gyy(:,:,k+2)
            lg(YZ,:,:,kpp) = ADM_BS_gyz(:,:,k+2)
            lg(ZZ,:,:,kpp) = ADM_BS_gzz(:,:,k+2)

            lA(XX,:,:,kpp) = ADM_BS_Axx(:,:,k+2)
            lA(XY,:,:,kpp) = ADM_BS_Axy(:,:,k+2)
            lA(XZ,:,:,kpp) = ADM_BS_Axz(:,:,k+2)
            lA(YY,:,:,kpp) = ADM_BS_Ayy(:,:,k+2)
            lA(YZ,:,:,kpp) = ADM_BS_Ayz(:,:,k+2)
            lA(ZZ,:,:,kpp) = ADM_BS_Azz(:,:,k+2)
         endif
#endif

c        CASE I: SHIFT IS ACTIVE.
         if (shift_state .ne. 0) then

c           Loop over x and y.
            do j=imin(2),imax(2)
               do i=imin(1),imax(1)

                  local_spatial_order = spatial_order
                  if (force_spatial_second_order(i,j,k)) local_spatial_order = 2
c
c     Note that there should be a check here
c     to see that storage for beta has been allocated;
c     it''s off by default.
c
                  BETAX_TEMP = betax(i,j,k)*i2dx
                  BETAY_TEMP = betay(i,j,k)*i2dy
                  BETAZ_TEMP = betaz(i,j,k)*i2dz

                  sx = 1; sy = 1; sz = 1
                  if (BETAX_TEMP < 0) sx = -1
                  if (BETAY_TEMP < 0) sy = -1
                  if (BETAZ_TEMP < 0) sz = -1

                  ssx = sx; ssy = sy; ssz = sz

c
c     Set flags so that admmacros use second-order upwind by default.
c
 
c                 if (doupwind1x(i,j,k)) ssx = 0
c                 if (doupwind1y(i,j,k)) ssy = 0
c                 if (doupwind1z(i,j,k)) ssz = 0

                  wx = 1; wy = 1; wz = 1

                  if (docenterx(i,j,k)) then
                     wx = 0; ssx = 0
                  end if
                  if (docentery(i,j,k)) then
                     wy = 0; ssy = 0
                  end if
                  if (docenterz(i,j,k)) then
                     wz = 0; ssz = 0
                  end if

#ifdef OPT

c                 Set ks based on shift direction at this point in space
c                 If the plethora of ks gets to be a problem then use
c                 preprocessor macros in place of all these, so that one
c                 can change them all at once if necessary.

                  ks  = 1 + mod(k+sz,5)
                  kss = 1 + mod(k+ssz,5)
                  ksss = 1 + mod(k+sz+ssz,5)

#endif

#include "macro/PHISOURCES_guts.h"
#include "macro/LIEPHI_guts.h"

                  adm_bs_sphi(i,j,k) = adm_bs_sphi(i,j,k) + LIEPHI_LPHI

#include "macro/BSGSOURCES_guts.h"
#include "macro/LIEBSG_guts.h"

                  adm_bs_sgxx(i,j,k) = adm_bs_sgxx(i,j,k) + LIEBSG_LGXX
                  adm_bs_sgxy(i,j,k) = adm_bs_sgxy(i,j,k) + LIEBSG_LGXY
                  adm_bs_sgxz(i,j,k) = adm_bs_sgxz(i,j,k) + LIEBSG_LGXZ
                  adm_bs_sgyy(i,j,k) = adm_bs_sgyy(i,j,k) + LIEBSG_LGYY
                  adm_bs_sgyz(i,j,k) = adm_bs_sgyz(i,j,k) + LIEBSG_LGYZ
                  adm_bs_sgzz(i,j,k) = adm_bs_sgzz(i,j,k) + LIEBSG_LGZZ

                  if (calcAdjustment) then

                    aux = Ksd * alp(i,j,k) * constraint_ham(i,j,k)

                    adm_bs_sgxx(i,j,k) = adm_bs_sgxx(i,j,k) + 
     +                   aux * adm_bs_gxx(i,j,k)

                    adm_bs_sgxy(i,j,k) = adm_bs_sgxy(i,j,k) + 
     +                   aux * adm_bs_gxy(i,j,k)

                    adm_bs_sgxz(i,j,k) = adm_bs_sgxz(i,j,k) + 
     +                   aux * adm_bs_gxz(i,j,k)

                    adm_bs_sgyy(i,j,k) = adm_bs_sgyy(i,j,k) + 
     +                   aux * adm_bs_gyy(i,j,k)

                    adm_bs_sgyz(i,j,k) = adm_bs_sgyz(i,j,k) + 
     +                   aux * adm_bs_gyz(i,j,k)

                    adm_bs_sgzz(i,j,k) = adm_bs_sgzz(i,j,k) + 
     +                   aux * adm_bs_gzz(i,j,k)

                  end if

               end do
            end do

c           Undefine macros.

#include "macro/PHISOURCES_undefine.h"
#include "macro/LIEPHI_undefine.h"

#include "macro/BSGSOURCES_undefine.h"
#include "macro/LIEBSG_undefine.h"

c        CASE II:  SHIFT IS OFF.
         else

c           Loop over x and y.

            do j=1+stencily,ny-stencily
               do i=2,nx-1

                  local_spatial_order = spatial_order
                  if (force_spatial_second_order(i,j,k)) local_spatial_order = 2

#include "macro/PHISOURCES_guts.h"
#include "macro/BSGSOURCES_guts.h"

               end do
            end do

c           Undefine macros.

#include "macro/PHISOURCES_undefine.h"
#include "macro/BSGSOURCES_undefine.h"

         end if
      end do

c=====================================================================
c
c     Construct sources for for K, Aij and Gammas.
c
c=====================================================================

#ifdef OPT
c     Initialize arrays for planes.

      lg(XX,:,:,2:4) = ADM_BS_gxx(:,:,1:3)
      lg(XY,:,:,2:4) = ADM_BS_gxy(:,:,1:3)
      lg(XZ,:,:,2:4) = ADM_BS_gxz(:,:,1:3)
      lg(YY,:,:,2:4) = ADM_BS_gyy(:,:,1:3)
      lg(YZ,:,:,2:4) = ADM_BS_gyz(:,:,1:3)
      lg(ZZ,:,:,2:4) = ADM_BS_gzz(:,:,1:3)

      lA(XX,:,:,2:4) = ADM_BS_Axx(:,:,1:3)
      lA(XY,:,:,2:4) = ADM_BS_Axy(:,:,1:3)
      lA(XZ,:,:,2:4) = ADM_BS_Axz(:,:,1:3)
      lA(YY,:,:,2:4) = ADM_BS_Ayy(:,:,1:3)
      lA(YZ,:,:,2:4) = ADM_BS_Ayz(:,:,1:3)
      lA(ZZ,:,:,2:4) = ADM_BS_Azz(:,:,1:3)
#endif

c     Loop over z.
      do k=imin(3),imax(3)

#ifdef OPT

c        Shift location in 'l' 'slabs'.

         km  = 1 + mod(k-1, 5)
         kc  = 1 + mod(k,   5)
         kp  = 1 + mod(k+1, 5)
         kpp = 1 + mod(k+2, 5)

c        If not the last k, put next slice into lg.

         if (k.ne.nz-1) then 
            lg(XX,:,:,kpp) = ADM_BS_gxx(:,:,k+2)
            lg(XY,:,:,kpp) = ADM_BS_gxy(:,:,k+2)
            lg(XZ,:,:,kpp) = ADM_BS_gxz(:,:,k+2)
            lg(YY,:,:,kpp) = ADM_BS_gyy(:,:,k+2)
            lg(YZ,:,:,kpp) = ADM_BS_gyz(:,:,k+2)
            lg(ZZ,:,:,kpp) = ADM_BS_gzz(:,:,k+2)

            lA(XX,:,:,kpp) = ADM_BS_Axx(:,:,k+2)
            lA(XY,:,:,kpp) = ADM_BS_Axy(:,:,k+2)
            lA(XZ,:,:,kpp) = ADM_BS_Axz(:,:,k+2)
            lA(YY,:,:,kpp) = ADM_BS_Ayy(:,:,k+2)
            lA(YZ,:,:,kpp) = ADM_BS_Ayz(:,:,k+2)
            lA(ZZ,:,:,kpp) = ADM_BS_Azz(:,:,k+2)
         endif
#endif

c        CASE I: SHIFT IS ACTIVE.

         if (shift_state .ne. 0) then

c           Loop over x and y.

            do j=imin(2),imax(2)
               do i=imin(1),imax(1)

                  local_spatial_order = spatial_order
                  if (force_spatial_second_order(i,j,k)) local_spatial_order = 2

                  BETAX_TEMP = betax(i,j,k)*i2dx
                  BETAY_TEMP = betay(i,j,k)*i2dy
                  BETAZ_TEMP = betaz(i,j,k)*i2dz

                  sx = 1; sy = 1; sz = 1
                  if (BETAX_TEMP < 0) sx = -1
                  if (BETAY_TEMP < 0) sy = -1
                  if (BETAZ_TEMP < 0) sz = -1

                  ssx = sx; ssy = sy; ssz = sz

c                 Set flags so that admmacros use second-order upwind by default.

c                 if (doupwind1x(i,j,k)) ssx = 0
c                 if (doupwind1y(i,j,k)) ssy = 0
c                 if (doupwind1z(i,j,k)) ssz = 0

                  wx = 1; wy = 1; wz = 1

                  if (docenterx(i,j,k)) then
                     wx = 0; ssx = 0
                  end if
                  if (docentery(i,j,k)) then
                     wy = 0; ssy = 0
                  end if
                  if (docenterz(i,j,k)) then
                     wz = 0; ssz = 0
                  end if

#ifdef OPT

c                 Set ks based on shift direction at this point in space
c                 If the plethora of ks gets to be a problem then use
c                 preprocessor macros in place of all these, so that one
c                 can change them all at once if necessary.

                  ks  = 1 + mod(k+sz,5)
                  kss = 1 + mod(k+ssz,5)
                  ksss = 1 + mod(k+sz+ssz,5)
#endif

#include "macro/TRKSOURCES_guts.h"
#include "macro/LIEK_guts.h"

                  adm_bs_sK(i,j,k) = adm_bs_sK(i,j,k) + LIEK_LK

                  if (calcAdjustment) then
                    adm_bs_sK(i,j,k) = adm_bs_sK(i,j,k) +
     +                   Kkm * alp(i,j,k) * constraint_DMom(i,j,k)
                  end if

#include "macro/ASOURCES_guts.h"
#include "macro/LIEA_guts.h"

                  adm_bs_sAxx(i,j,k) = adm_bs_sAxx(i,j,k) + LIEA_LAXX
                  adm_bs_sAxy(i,j,k) = adm_bs_sAxy(i,j,k) + LIEA_LAXY
                  adm_bs_sAxz(i,j,k) = adm_bs_sAxz(i,j,k) + LIEA_LAXZ
                  adm_bs_sAyy(i,j,k) = adm_bs_sAyy(i,j,k) + LIEA_LAYY
                  adm_bs_sAyz(i,j,k) = adm_bs_sAyz(i,j,k) + LIEA_LAYZ
                  adm_bs_sAzz(i,j,k) = adm_bs_sAzz(i,j,k) + LIEA_LAZZ

                  if (calcAdjustment) then
                    aux1 = K8 * alp(i,j,k) * exp(-4*ADM_BS_phi(i,j,k)) *
     +                   constraint_PG(i,j,k)
                    aux2 = Kam1 * alp(i,j,k) * constraint_DMom(i,j,k)
                    aux3 = Kam2 * alp(i,j,k)

                    adm_bs_sAxx(i,j,k) = adm_bs_sAxx(i,j,k) +
     +                   (aux1 + aux2) * ADM_BS_gxx(i,j,k) +
     +                   aux3 * constraint_DMom_xx(i,j,k)
                    adm_bs_sAxy(i,j,k) = adm_bs_sAxy(i,j,k) +
     +                   (aux1 + aux2) * ADM_BS_gxy(i,j,k) +
     +                   aux3 * constraint_DMom_xy(i,j,k)
                    adm_bs_sAxz(i,j,k) = adm_bs_sAxz(i,j,k) +
     +                   (aux1 + aux2) * ADM_BS_gxz(i,j,k) +
     +                   aux3 * constraint_DMom_xz(i,j,k)
                    adm_bs_sAyy(i,j,k) = adm_bs_sAyy(i,j,k) +
     +                   (aux1 + aux2) * ADM_BS_gyy(i,j,k) +
     +                   aux3 * constraint_DMom_yy(i,j,k)
                    adm_bs_sAyz(i,j,k) = adm_bs_sAyz(i,j,k) +
     +                   (aux1 + aux2) * ADM_BS_gyz(i,j,k) +
     +                   aux3 * constraint_DMom_yz(i,j,k)
                    adm_bs_sAzz(i,j,k) = adm_bs_sAzz(i,j,k) +
     +                   (aux1 + aux2) * ADM_BS_gzz(i,j,k) +
     +                   aux3 * constraint_DMom_zz(i,j,k)

                  end if

#include "macro/GAMMASOURCES_guts.h"
#include "macro/SHIFTGAMMA_guts.h"

                  adm_bs_sGx(i,j,k) = adm_bs_sGx(i,j,k) + SHIFTGAMMAX
                  adm_bs_sGy(i,j,k) = adm_bs_sGy(i,j,k) + SHIFTGAMMAY
                  adm_bs_sGz(i,j,k) = adm_bs_sGz(i,j,k) + SHIFTGAMMAZ

                  if (calcAdjustment) then
                     aux = (betax(i+1,j,k) - betax(i-1,j,k))*0.5*idx +
     +               (betay(i,j+1,k) - betay(i,j-1,k))*0.5*idy +
     +               (betaz(i,j,k+1) - betaz(i,j,k-1))*0.5*idz

                     bssn_divbeta(i,j,k) = aux

                     adm_bs_sGx(i,j,k) = adm_bs_sGx(i,j,k)-baumgarte_chi*
     +               constraint_Gx(i,j,k)*aux
                     adm_bs_sGy(i,j,k) = adm_bs_sGy(i,j,k)-baumgarte_chi*
     +               constraint_Gy(i,j,k)*aux
                     adm_bs_sGz(i,j,k) = adm_bs_sGz(i,j,k)-baumgarte_chi*
     +               constraint_Gz(i,j,k)*aux
                  end if

               end do
            end do

c           Undefine macros.

#include "macro/TRKSOURCES_undefine.h"
#include "macro/LIEK_undefine.h"

#include "macro/ASOURCES_undefine.h"
#include "macro/LIEA_undefine.h"

#include "macro/GAMMASOURCES_undefine.h"
#include "macro/SHIFTGAMMA_undefine.h"

c        CASE II: SHIFT IS OFF.

         else

c           Loop over x and y.

            do j=1+stencily,ny-stencily
               do i=2,nx-1

                  local_spatial_order = spatial_order
                  if (force_spatial_second_order(i,j,k)) local_spatial_order = 2

#include "macro/TRKSOURCES_guts.h"
#include "macro/ASOURCES_guts.h"
#include "macro/GAMMASOURCES_guts.h"

               end do
            end do

c           Undefine macros.

#include "macro/TRKSOURCES_undefine.h"
#include "macro/ASOURCES_undefine.h"
#include "macro/GAMMASOURCES_undefine.h"

         end if
      end do


c=========================================================================
c
c     Construct sources for lapse and shift and attenuate.
c
c=========================================================================

c     Optionally attenuate sources with function of Hamiltonian constraint.
c     Under development (see John).

      if (attenuate.eq.1) then
        call ADM_BSSN_AttenuateSource(CCTK_ARGUMENTS)
      end if

c     Sources for lapse if needed.  Must be done after phi and trK.

      if (evolveLapse) then
         call ADM_BSSN_LapseSource(CCTK_ARGUMENTS)
      end if

c     Sources for shift if needed.  Must be done after metric and Gammas.

      if (evolveShift) then
         call ADM_BSSN_ShiftSource(CCTK_ARGUMENTS)
      end if

c     Optionally attenuate gauge evolution (see John).

      if (attenuate_gauge.eq.1)then
        call ADM_BSSN_AttenuateGauge(CCTK_ARGUMENTS)
      end if

c========================================================================
c
c     Deallocate arrays.
c
c========================================================================
      deallocate(force_spatial_second_order)

      if (shift_state .ne. 0) then
         deallocate(bigmask)
c        deallocate(doupwind1x,doupwind1y,doupwind1z)
         deallocate(docenterx,docentery,docenterz)
      end if

#ifdef OPT
      deallocate(lg,STAT=allocstat)
      deallocate(lA,STAT=allocstat)
#undef OPT
#endif

c========================================================================
c
c     End.
c
c========================================================================

      end subroutine ADM_BSSN_Sources
#undef INSOURCES
